@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -20,16 +20,9 @@
 
 
 
-#if defined(AFS_HPUX110_ENV) && !defined(__HP_CURSES)
-#define __HP_CURSES
-#endif
-
-#ifndef	AFS_SUN5_ENV
-#include <curses.h>		/*Curses library */
-#endif
 #include <sys/types.h>
 #include <sys/file.h>
-#if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV)
+#if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD80_ENV)
 #include <sgtty.h>
 #endif
 #include <stdio.h>
@@ -578,6 +571,7 @@ gator_cursesgwin_drawchar(struct gwin *gwp, struct gwin_charparams *params)
     static char rn[] = "gator_cursesgwin_drawchar";	/*Routine name */
     struct gator_cursesgwin *cwp;	/*Ptr to curses private area */
     int curses_x, curses_y;	/*Mapped x,y positions */
+    int code=0;
 
     cwp = (struct gator_cursesgwin *)(gwp->w_data);
     curses_x = GATOR_MAP_X_TO_COL(cwp, params->x);
@@ -588,11 +582,17 @@ gator_cursesgwin_drawchar(struct gwin *gwp, struct gwin_charparams *params)
 		mn, rn, params->c, cwp->wp, params->x, params->y, curses_y,
 		curses_x, (params->highlight ? ", using standout mode" : ""));
     wmove(cwp->wp, curses_y, curses_x);
-    if (params->highlight)
-	wstandout(cwp->wp);
+    if (params->highlight) {
+	code=wstandout(cwp->wp);
+        if (code)
+	    return (code);
+    }
     waddch(cwp->wp, params->c);
-    if (params->highlight)
-	wstandend(cwp->wp);
+    if (params->highlight) {
+	code=wstandend(cwp->wp);
+        if (code)
+            return (code);
+    }
 
     return (0);
 
@@ -627,6 +627,7 @@ gator_cursesgwin_drawstring(struct gwin *gwp, struct gwin_strparams *params)
     static char rn[] = "gator_cursesgwin_drawstring";	/*Routine name */
     struct gator_cursesgwin *cwp;	/*Ptr to curses private area */
     int curses_x, curses_y;	/*Mapped x,y positions */
+    int code=0;
 
     cwp = (struct gator_cursesgwin *)(gwp->w_data);
     curses_x = GATOR_MAP_X_TO_COL(cwp, params->x);
@@ -637,13 +638,19 @@ gator_cursesgwin_drawstring(struct gwin *gwp, struct gwin_strparams *params)
 		mn, rn, params->s, cwp->wp, params->x, params->y, curses_y,
 		curses_x, (params->highlight ? ", using standout mode" : ""));
     wmove(cwp->wp, curses_y, curses_x);
-    if (params->highlight)
-	wstandout(cwp->wp);
+    if (params->highlight) {
+	code=wstandout(cwp->wp);
+	if (code)
+	    return (code);
+    }
     waddstr(cwp->wp, params->s);
-    if (params->highlight)
-	wstandend(cwp->wp);
+    if (params->highlight) {
+	code=wstandend(cwp->wp);
+	if (code)
+	    return (code);
+    }
 
-    return (0);
+    return (code);
 
 }				/*gator_cursesgwin_drawstring */
 
@@ -769,16 +776,7 @@ gator_cursesgwin_getdimensions(struct gwin *gwp, struct gwin_sizeparams *aparms)
     struct gator_cursesgwin *cwp;	/*Curses-specific data */
 
     cwp = (struct gator_cursesgwin *)(gwp->w_data);
-#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN60_ENV)
-    aparms->maxx = cwp->wp->maxx;
-    aparms->maxy = cwp->wp->maxy;
-#elif defined(AFS_NBSD_ENV) || defined(AFS_DARWIN100_ENV)
-    aparms->maxx = getmaxx(cwp->wp);
-    aparms->maxy = getmaxy(cwp->wp);
-#else
-    aparms->maxx = cwp->wp->_maxx;
-    aparms->maxy = cwp->wp->_maxy;
-#endif
+    getmaxyx(cwp->wp, aparms->maxy, aparms->maxx);
 
     return (0);
 
