@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -51,11 +51,8 @@ static FILE *childin, *childout;	/* file pointers on pipe to kpwvalid */
 static int
 simplify_name(char *orig_name, char *true_name)
 {
-    int thru_symlink;
     struct stat statbuff;
 
-    thru_symlink = 0;
-
 #ifdef AFS_NT40_ENV
     if (stat(orig_name, &statbuff) < 0) {
 	*true_name = '\0';
@@ -69,7 +66,7 @@ simplify_name(char *orig_name, char *true_name)
 	int link_chars_read;
 	char *last_component;
 	if (lstat(orig_name, &statbuff) < 0) {
-	    /* if lstat fails, it's possible that it's transient, but 
+	    /* if lstat fails, it's possible that it's transient, but
 	     * unlikely.  Let's hope it isn't, and continue... */
 	    *true_name = '\0';
 	    return 0;
@@ -80,7 +77,6 @@ simplify_name(char *orig_name, char *true_name)
 	 * the contents of the link for the file name.
 	 */
 	if ((statbuff.st_mode & S_IFMT) == S_IFLNK) {
-	    thru_symlink = 1;
 	    link_chars_read = readlink(orig_name, true_name, 1024);
 	    if (link_chars_read <= 0) {
 		*true_name = '\0';
@@ -117,7 +113,7 @@ simplify_name(char *orig_name, char *true_name)
    * if it exists, it must be us, eh?
    * NB there may be possible security implications involving
    * symlinks; I think they are only relevant if the symlink points
-   * directly at kpasswd, not when it points at kpasswd's parent directory. 
+   * directly at kpasswd, not when it points at kpasswd's parent directory.
    */
 static int
 find_me(char *arg, char *parent_dir)
@@ -206,10 +202,10 @@ find_me(char *arg, char *parent_dir)
 
 /* this function returns TRUE (1) if the file is in AFS, otherwise false (0) */
 static int
-InAFS(register char *apath)
+InAFS(char *apath)
 {
     struct ViceIoctl blob;
-    register afs_int32 code;
+    afs_int32 code;
     char space[AFS_PIOCTL_MAXSIZE];
 
     blob.in_size = 0;
@@ -294,8 +290,8 @@ safestrtok(char *str, char *tok)
  * there's not much point in being more secure than the kpasswd executable.
  */
 /* 1.  is this directory in AFS?
- * 2.  Is every component of the pathname secure 
- *     (ie, only system:administrators have w or a rights)? 
+ * 2.  Is every component of the pathname secure
+ *     (ie, only system:administrators have w or a rights)?
  */
 static int
 is_secure(char *dir)
@@ -312,7 +308,7 @@ is_secure(char *dir)
 
 #ifndef INSECURE
     for (temp = safestrtok(dir, "/"); temp; temp = safestrtok(NULL, "/")) {
-	/* strtok keeps sticking NUL in place of /, so we can look at 
+	/* strtok keeps sticking NUL in place of /, so we can look at
 	 * ever-longer chunks of the path.
 	 */
 	if (!InAFS(dir))
@@ -342,7 +338,7 @@ is_secure(char *dir)
 }
 
 /* Then, once we've found our own location, we look for a program named
- * kpwvalid.  
+ * kpwvalid.
  */
 
 /* look for a password-checking program named kpwvalid.
@@ -358,7 +354,7 @@ kpwvalid_is(char *dir)
     strcpy(dir + len, "/kpwvalid");
 
     if (stat(dir, &statbuff) < 0) {
-	/* if lstat fails, it's possible that it's transient, but 
+	/* if lstat fails, it's possible that it's transient, but
 	 * unlikely.  Let's hope it isn't, and continue... */
 	*(dir + len) = '\0';
 	return 0;
@@ -370,7 +366,7 @@ kpwvalid_is(char *dir)
 
 #ifdef AFS_NT40_ENV
 /* We don't allow the use of kpwvalid executable scripts to set policy
- * for passwd changes. 
+ * for passwd changes.
  */
 int
 init_child(char *myname)
@@ -400,8 +396,10 @@ init_child(char *myname)
      * reads from the latter, the child reads from the former, and
      * writes to the latter.
      */
-    pipe(pipe1);
-    pipe(pipe2);
+    if (pipe(pipe1) || pipe(pipe2)) {
+	using_child = 0;
+	return 0;
+    }
 
     /* fork a child */
     pid = fork();
@@ -442,7 +440,8 @@ password_bad(char *pw)
     if (using_child) {
 	fprintf(childin, "%s\n", pw);
 	fflush(childin);
-	fscanf(childout, "%d", &rc);
+	if (fscanf(childout, "%d", &rc) < 1)
+	    rc = -1;
     }
 
     return (rc);
