@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -11,7 +11,6 @@
  * This file implements the kas related funtions for afscp
  */
 
-#include <afs/param.h>
 #include <afs/stds.h>
 
 #include <stddef.h>
@@ -22,6 +21,9 @@
 
 #include <pthread.h>
 
+#include <rx/rx.h>
+#include <rx/rxstat.h>
+
 #include <afs/afs_Admin.h>
 #include <afs/afs_kasAdmin.h>
 #include <afs/afs_utilAdmin.h>
@@ -32,4 +34,4 @@
 #include "common.h"
 
 void
-SetupKasAdminCmd(void);
+  SetupKasAdminCmd(void);
