@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vlserver/vlclient.c,v 1.16.2.5 2009/03/23 18:19:06 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -39,6 +37,7 @@ RCSID
 #endif
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 
 #include <afs/afsutil.h>
 #include <rx/xdr.h>
@@ -51,18 +50,17 @@ RCSID
 #include <lock.h>
 #include <ubik.h>
 #include "vlserver.h"
-#include "vlclient.h"
 
-void fill_listattributes_entry();
-void display_listattributes_entry();
-void display_entry();
-void display_entryN();
-void display_update_entry();
-void dump_stats();
-void GetArgs();
-void print_usage();
-void fill_entry();
-void fill_update_entry();
+void fill_listattributes_entry(struct VldbListByAttributes *, char **, int);
+void display_listattributes_entry(struct VldbListByAttributes *,int);
+void display_entry(struct vldbentry *, int);
+void display_entryN(struct nvldbentry *, int);
+void display_update_entry(struct VldbUpdateEntry *, int);
+void dump_stats(vldstats *, vital_vlheader *);
+void GetArgs(char *, char **, int *);
+void print_usage(void);
+void fill_entry(struct vldbentry *, char **, int);
+void fill_update_entry(struct VldbUpdateEntry *, char **, int);
 
 #define	VL_NUMBER_OPCODESX	34
 static char *opcode_names[VL_NUMBER_OPCODESX] = {
@@ -79,7 +77,7 @@ static char *opcode_names[VL_NUMBER_OPCODESX] = {
     "ListAttributes",
     "LinkedList",
     "GetStats",
-    "Probe",
+    "Probe",	/* RPC is ProbeServer */
     "GetAddrs",
     "ChangeAddr",
     "CreateEntryN",
@@ -122,11 +120,10 @@ char confdir[AFSDIR_PATH_MAX];
 char *(args[50]);
 
 struct Vlent *
-GetVolume(vol, entry)
-     struct vldbentry *entry;
+GetVolume(int vol, struct vldbentry *entry)
 {
-    register int i;
-    register struct Vlent *vl;
+    int i;
+    struct Vlent *vl;
 
     if (!vol)
 	return NULL;
@@ -157,7 +154,7 @@ afs_int32
 vl_Initialize(int auth, char *confDir, int server, char *cellp)
 {
     return ugen_ClientInit(auth?0:1, confDir, cellp, 0,
-			  &cstruct, NULL, "vl_Initialize", rxkad_clear, 
+			  &cstruct, NULL, "vl_Initialize", rxkad_clear,
 			  MAXSERVERS, AFSCONF_VLDBSERVICE, 50, server,
 			  htons(AFSCONF_VLDBPORT), USER_SERVICE_ID);
 }
@@ -166,10 +163,10 @@ vl_Initialize(int auth, char *confDir, int server, char *cellp)
 afs_int32
 GetServer(char *aname)
 {
-    register struct hostent *th;
-    afs_int32 addr;
+    struct hostent *th;
+    afs_uint32 addr;
     int b1, b2, b3, b4;
-    register afs_int32 code;
+    afs_int32 code;
 
     code = sscanf(aname, "%d.%d.%d.%d", &b1, &b2, &b3, &b4);
     if (code == 4) {
@@ -187,22 +184,22 @@ GetServer(char *aname)
 static int
 handleit(struct cmd_syndesc *as, void *arock)
 {
-    register struct cmd_item *ti;
-    register afs_int32 code, server = 0, sawserver = 0;
+    struct cmd_item *ti;
+    afs_int32 code, server = 0, sawserver = 0;
     afs_int32 id, voltype;
     struct vldbentry entry;
-    char *cmd = 0, *cellp = 0;
+    char *cellp = 0;
     struct VldbUpdateEntry updateentry;
     struct VldbListByAttributes listbyattributes;
     int noAuth = 1;		/* Default is authenticated connections */
 
-    if (ti = as->parms[0].items)	/* -cellpath <dir> */
+    if ((ti = as->parms[0].items))	/* -cellpath <dir> */
 	strcpy(confdir, ti->data);
     if (as->parms[1].items)	/* -server */
 	strcpy(confdir, AFSDIR_SERVER_ETC_DIRPATH);
     if (as->parms[2].items)	/* -noauth */
 	noAuth = 0;
-    if (ti = as->parms[3].items) {	/* -host */
+    if ((ti = as->parms[3].items)) {	/* -host */
 	server = GetServer(ti->data);
 	if (server == 0) {
 	    printf("server '%s' not found in host table\n", ti->data);
@@ -215,15 +212,50 @@ handleit(struct cmd_syndesc *as, void *arock)
 	    ("Must also specify the -cell' option along with -host for authenticated conns\n");
 	exit(1);
     }
-    if (ti = as->parms[4].items) {	/* -cell */
+    if ((ti = as->parms[4].items)) {	/* -cell */
 	cellp = ti->data;
     }
-    if (code = vl_Initialize(noAuth, confdir, server, cellp)) {
+    if ((code = vl_Initialize(noAuth, confdir, server, cellp))) {
 	printf("Couldn't initialize vldb library (code=%d).\n", code);
 	exit(1);
     }
 
-    if (as->parms[5].items) {	/* -gstats */
+    if (as->parms[6].items) {	/* -probe */
+	int nconns = 0;
+	int status = 0;
+	int i;
+	char hoststr[16];
+	afs_uint32 addr;
+
+	for (i = 0; cstruct->conns[i]; i++, nconns++) {
+	    rx_SetConnDeadTime(cstruct->conns[i], 6);
+	    addr = rx_HostOf(rx_PeerOf(cstruct->conns[i]));
+	    if (!nconns) {
+		printf("probing");
+	    }
+	    printf(" %s", afs_inet_ntoa_r(addr, hoststr));
+	}
+	if (nconns == 0) {
+	    printf("no connections\n");
+	    return 255;
+	}
+	printf("\n");
+	multi_Rx(cstruct->conns, nconns) {
+	    multi_VL_ProbeServer();
+	    addr = rx_HostOf(rx_PeerOf(cstruct->conns[multi_i]));
+	    if (!multi_error) {
+		printf(" ok: %s\n", afs_inet_ntoa_r(addr, hoststr));
+	    } else {
+		status = 255;
+		printf(" not ok (%d): %s\n", multi_error,
+		       afs_inet_ntoa_r(addr, hoststr));
+	    }
+	}
+	multi_End;
+	return status;
+    }
+
+    if (as->parms[5].items) {	/* -getstats */
 	vldstats stats;
 	vital_vlheader vital_header;
 	code = ubik_VL_GetStats(cstruct, 0, &stats, &vital_header);
@@ -244,11 +276,13 @@ handleit(struct cmd_syndesc *as, void *arock)
 	    exit(0);
 	} else {
 	    char *oper, *vname;
-	    register char **argp = args;
+	    char **argp = args;
+
 	    GetArgs(line, argp, &nargs);
 	    oper = &argp[0][0];
 	    ++argp, --nargs;
-	    if (!strcmp(oper, "cr")) {
+	    if (!*line) {
+	    } else if (!strcmp(oper, "cr")) {
 		fill_entry(&entry, argp, nargs);
 		display_entry(&entry, 0);
 		code = ubik_VL_CreateEntry(cstruct, 0, &entry);
@@ -324,21 +358,21 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    if (!next_index)
 			break;
 		    num++;
-		    if (vl1 = GetVolume(entry.volumeId[0], &entry)) {
+		    if ((vl1 = GetVolume(entry.volumeId[0], &entry))) {
 			num1++;
 			printf
 			    ("Duplicate entry is found for RW vol %u: [RW %u, RO %u, BA %u, name=%s]\n",
 			     entry.volumeId[0], vl1->rwid, vl1->roid,
 			     vl1->baid, vl1->name);
 		    }
-		    if (vl1 = GetVolume(entry.volumeId[1], &entry)) {
+		    if ((vl1 = GetVolume(entry.volumeId[1], &entry))) {
 			num1++;
 			printf
 			    ("Duplicate entry is found for RO vol %u: [RW %u, RO %u, BA %u, name=%s]\n",
 			     entry.volumeId[1], vl1->rwid, vl1->roid,
 			     vl1->baid, vl1->name);
 		    }
-		    if (vl1 = GetVolume(entry.volumeId[2], &entry)) {
+		    if ((vl1 = GetVolume(entry.volumeId[2], &entry))) {
 			num1++;
 			printf
 			    ("Duplicate entry is found for BA vol %u: [RW %u, RO %u, BA %u, name=%s]\n",
@@ -437,8 +471,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    ("Volumes not found in main hash tables in vldb will be fixed...\n");
 		memset(&updateentry, 0, sizeof(updateentry));
 		for (index = 0; 1; index = next_index) {
-		    /* FIXME: n2 is never changed for some reason */
-		    int n1 = 0, n2 = 0, n3 = 0, n4 = 0;
 		    memset(&entry, 0, sizeof(entry));
 		    code =
 			ubik_VL_ListEntry(cstruct, 0, index, &count,
@@ -455,7 +487,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 				  &tentry);
 		    if (code == VL_NOENT) {
 			num1++;
-			n1 = 1;
 			updateentry.Mask = VLUPDATE_VOLNAMEHASH;
 			printf("\tVolume %s %d (not in namehash)\n",
 			       entry.name, entry.volumeId[RWVOL]);
@@ -489,13 +520,12 @@ handleit(struct cmd_syndesc *as, void *arock)
 			}
 			x++;
 		    }
-		    if (entry.volumeId[BACKVOL] && !n2) {
+		    if (entry.volumeId[BACKVOL]) {
 			code =
 			    ubik_VL_GetEntryByID(cstruct, 0,
 				      entry.volumeId[BACKVOL], BACKVOL,
 				      &tentry);
 			if (code == VL_NOENT) {
-			    n3 = 1;
 			    num1++;
 			    updateentry.Mask = VLUPDATE_BACKUPID;
 			    updateentry.BackupId = entry.volumeId[BACKVOL];
@@ -513,12 +543,11 @@ handleit(struct cmd_syndesc *as, void *arock)
 			    }
 			}
 		    }
-		    if (entry.volumeId[ROVOL && !n2]) {
+		    if (entry.volumeId[ROVOL]) {
 			code =
 			    ubik_VL_GetEntryByID(cstruct, 0,
 				      entry.volumeId[ROVOL], ROVOL, &tentry);
 			if (code == VL_NOENT) {
-			    n4 = 1;
 			    num1++;
 			    updateentry.Mask = VLUPDATE_READONLYID;
 			    updateentry.ReadOnlyId = entry.volumeId[ROVOL];
@@ -618,8 +647,8 @@ handleit(struct cmd_syndesc *as, void *arock)
 		       (netries == 1 ? "y" : "ies"));
 		for (vllist = linkedvldbs.node; vllist; vllist = vllist1) {
 		    vllist1 = vllist->next_vldb;
-		    display_entry(&vllist->VldbEntry, 0);
-		    free((char *)vllist);
+		    display_entry((struct vldbentry *) &vllist->VldbEntry, 0);
+		    free(vllist);
 		}
 	    } else if (!strcmp(oper, "lnn")) {
 		int netries;
@@ -640,7 +669,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 		       (netries == 1 ? "y" : "ies"));
 		for (vllist = linkedvldbs.node; vllist; vllist = vllist1) {
 		    vllist1 = vllist->next_vldb;
-		    display_entry(&vllist->VldbEntry, 0);
+		    display_entry((struct vldbentry *)&vllist->VldbEntry, 0);
 		    free((char *)vllist);
 		}
 	    } else if (!strcmp(oper, "di")) {
@@ -728,12 +757,12 @@ handleit(struct cmd_syndesc *as, void *arock)
 		display_entry(&entry, code);
 		printf("return code is %d.\n", code);
 	    } else if (!strcmp(oper, "nv")) {
-		int newvolid;
+		unsigned int newvolid;
 		sscanf(&(*argp)[0], "%d", &id);
 		code =
 		    ubik_VL_GetNewVolumeId(cstruct, 0, id, &newvolid);
 		if (!code)
-		    printf("Current Max volid is (in hex):%X\n", newvolid);
+		    printf("Current Max volid is (in hex):0x%x\n", newvolid);
 		printf("return code is %d\n", code);
 	    } else if (!strcmp(oper, "gs")) {
 		vldstats stats;
@@ -795,6 +824,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 
 			printf("[0x%x %u] (special multi-homed entry)\n",
 			       *addrp, *addrp);
+			memset(&attrs, 0, sizeof(attrs));
 			attrs.Mask = VLADDR_INDEX;
 			mhaddrs.bulkaddrs_val = 0;
 			mhaddrs.bulkaddrs_len = 0;
@@ -831,7 +861,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 		}
 		free((char *)addrs.bulkaddrs_val);
 	    } else if (!strcmp(oper, "mhc")) {
-		afs_int32 serveraddrs[MAXSERVERID + 1][VL_MAXIPADDRS_PERMH];
+		afs_uint32 serveraddrs[MAXSERVERID + 1][VL_MAXIPADDRS_PERMH];
 		afs_int32 serveraddrtype[MAXSERVERID + 1];
 		int nentries1, nentries2, i, j, x, y, unique, found;
 		afs_uint32 *addrp1, *addrp2;
@@ -870,6 +900,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 
 			addrs2.bulkaddrs_val = 0;
 			addrs2.bulkaddrs_len = 0;
+			memset(&attrs, 0, sizeof(attrs));
 			attrs.Mask = VLADDR_INDEX;
 			attrs.index = (base * VL_MHSRV_PERBLK) + index;
 			code =
@@ -964,7 +995,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    continue;
 		}
 	    } else if (!strcmp(oper, "ca")) {
-		extern struct hostent *hostutil_GetHostByName();
 		struct hostent *h1, *h2;
 		afs_uint32 a1, a2;
 
@@ -1008,12 +1038,13 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    printf("VL_ChangeAddr returned code = %d\n", code);
 		    continue;
 		}
-	    } else if ((!strcmp(oper, "?")) || !strcmp(oper, "h"))
+	    } else if ((!strcmp(oper, "?")) || !strcmp(oper, "h") || !strcmp(oper, "help"))
 		print_usage();
 	    else if ((!strcmp(oper, "q")) || !strcmp(oper, "quit"))
 		exit(0);
 	    else {
-		printf("Unknown oper!\n");
+		printf("Unknown oper (%s)!\n", oper);
+		print_usage();
 	    }
 	}
     }
@@ -1022,11 +1053,10 @@ handleit(struct cmd_syndesc *as, void *arock)
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char **argv)
 {
-    register struct cmd_syndesc *ts;
+    struct cmd_syndesc *ts;
     afs_int32 code;
 
     strcpy(confdir, AFSDIR_CLIENT_ETC_DIRPATH);
@@ -1043,16 +1073,15 @@ main(argc, argv)
 		"cellname '-host' belongs to (required for auth conns)");
     cmd_AddParm(ts, "-getstats", CMD_FLAG, CMD_OPTIONAL,
 		"print vldb statistics (non interactive)");
+    cmd_AddParm(ts, "-probe", CMD_FLAG, CMD_OPTIONAL,
+		"probe vldb servers, use with -host to probe only one. (non interactive)");
     code = cmd_Dispatch(argc, argv);
     exit(code);
 }
 
 
 void
-fill_entry(entry, argp, nargs)
-     struct vldbentry *entry;
-     char **argp;
-     int nargs;
+fill_entry(struct vldbentry *entry, char **argp, int nargs)
 {
     char *name;
     int i;
@@ -1086,10 +1115,7 @@ fill_entry(entry, argp, nargs)
 }
 
 void
-fill_update_entry(entry, argp, nargs)
-     struct VldbUpdateEntry *entry;
-     char **argp;
-     int nargs;
+fill_update_entry(struct VldbUpdateEntry *entry, char **argp, int nargs)
 {
     int i, Mask;
     char *name;
@@ -1154,10 +1180,8 @@ fill_update_entry(entry, argp, nargs)
 }
 
 void
-fill_listattributes_entry(entry, argp, nargs)
-     struct VldbListByAttributes *entry;
-     char **argp;
-     int nargs;
+fill_listattributes_entry(struct VldbListByAttributes *entry, char **argp,
+			  int nargs)
 {
     entry->Mask = 0;
 
@@ -1191,9 +1215,7 @@ fill_listattributes_entry(entry, argp, nargs)
 }
 
 void
-display_listattributes_entry(entry, error)
-     struct VldbListByAttributes *entry;
-     int error;
+display_listattributes_entry(struct VldbListByAttributes *entry, int error)
 {
     if (error)
 	return;
@@ -1214,9 +1236,7 @@ display_listattributes_entry(entry, error)
 #define	volumetype_string(type) (type == RWVOL? "read/write":type == ROVOL? "readonly":type == BACKVOL? "backup":"unknown")
 
 void
-display_entry(entry, error)
-     struct vldbentry *entry;
-     int error;
+display_entry(struct vldbentry *entry, int error)
 {
     int i;
 
@@ -1235,9 +1255,7 @@ display_entry(entry, error)
 }
 
 void
-display_entryN(entry, error)
-     struct nvldbentry *entry;
-     int error;
+display_entryN(struct nvldbentry *entry, int error)
 {
     int i, et, ei;
 
@@ -1263,9 +1281,7 @@ display_entryN(entry, error)
 }
 
 void
-display_update_entry(entry, error)
-     struct VldbUpdateEntry *entry;
-     int error;
+display_update_entry(struct VldbUpdateEntry *entry, int error)
 {
     int i;
 
@@ -1279,19 +1295,19 @@ display_update_entry(entry, error)
     if (entry->Mask & VLUPDATE_CLONEID)
 	printf("\tNew CloneId: %X\n", entry->cloneId);
     if (entry->Mask & VLUPDATE_READONLYID)
-	printf("\tNew RO id: %D\n", entry->ReadOnlyId);
+	printf("\tNew RO id: %d\n", entry->ReadOnlyId);
     if (entry->Mask & VLUPDATE_BACKUPID)
-	printf("\tNew BACKUP id: %D\n", entry->BackupId);
+	printf("\tNew BACKUP id: %d\n", entry->BackupId);
     if (entry->Mask & VLUPDATE_REPSITES) {
 	printf("\tRepsites info:\n");
 	printf("\tFlag\tTServer\tTPart\tNServer\tNPart\tNFlag\n");
 	for (i = 0; i < entry->nModifiedRepsites; i++) {
-	    printf("\t%4x\t%7U\t%5d", entry->RepsitesMask[i],
+	    printf("\t%4x\t%7u\t%5d", entry->RepsitesMask[i],
 		   entry->RepsitesTargetServer[i],
 		   entry->RepsitesTargetPart[i]);
 	    if ((entry->RepsitesMask[i] & VLUPDATE_REPS_ADD)
 		|| (entry->RepsitesMask[i] & VLUPDATE_REPS_MODSERV))
-		printf("\t%7U", entry->RepsitesNewServer[i]);
+		printf("\t%7u", entry->RepsitesNewServer[i]);
 	    else
 		printf("\t-------");
 	    if ((entry->RepsitesMask[i] & VLUPDATE_REPS_ADD)
@@ -1309,15 +1325,13 @@ display_update_entry(entry, error)
 }
 
 void
-dump_stats(stats, vital_header)
-     vldstats *stats;
-     vital_vlheader *vital_header;
+dump_stats(vldstats *stats, vital_vlheader *vital_header)
 {
     int i;
     char strg[30];
     time_t start_time = stats->start_time;
 
-    strncpy(strg, ctime(&start_time), sizeof(strg));
+    afs_ctime(&start_time, strg, sizeof(strg));
     strg[strlen(strg) - 1] = 0;
     printf("Dynamic statistics stats (starting time: %s):\n", strg);
     printf("OpcodeName\t# Requests\t# Aborts\n");
@@ -1335,28 +1349,25 @@ dump_stats(stats, vital_header)
 }
 
 void
-GetArgs(line, args, nargs)
-     register char *line;
-     register char **args;
-     register int *nargs;
+GetArgs(char *line, char **args, int *nargs)
 {
     *nargs = 0;
     while (*line) {
-	register char *last = line;
-	while (*line == ' ')
+	char *last = line;
+	while (isspace(*line))
 	    line++;
-	if (*last == ' ')
+	if (isspace(*last))
 	    *last = 0;
 	if (!*line)
 	    break;
 	*args++ = line, (*nargs)++;
-	while (*line && *line != ' ')
+	while (*line && !isspace(*line))
 	    line++;
     }
 }
 
 void
-print_usage()
+print_usage(void)
 {
     printf("Valid Commands:\n");
 
