@@ -135,7 +135,10 @@ namei_iread(IHandle_t * h, afs_foff_t offset, char *buf, afs_fsize_t size)
 	return -1;
 
     nBytes = FDH_PREAD(fdP, buf, size, offset);
-    FDH_CLOSE(fdP);
+    if (nBytes < 0)
+	FDH_REALLYCLOSE(fdP);
+    else
+	FDH_CLOSE(fdP);
     return nBytes;
 }
 
@@ -150,7 +153,10 @@ namei_iwrite(IHandle_t * h, afs_foff_t offset, char *buf, afs_fsize_t size)
 	return -1;
 
     nBytes = FDH_PWRITE(fdP, buf, size, offset);
-    FDH_CLOSE(fdP);
+    if (nBytes < 0)
+	FDH_REALLYCLOSE(fdP);
+    else
+	FDH_CLOSE(fdP);
     return nBytes;
 }
 
@@ -372,7 +378,7 @@ int
 namei_ViceREADME(char *partition)
 {
     char filename[32];
-    int fd;
+    int fd, len, e = 0;
 
     /* Create the inode directory if we're starting for the first time */
     (void)afs_snprintf(filename, sizeof filename, "%s" OS_DIRSEP "%s", partition,
@@ -383,8 +389,13 @@ namei_ViceREADME(char *partition)
                        partition, INODEDIR);
     fd = afs_open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0444);
     if (fd >= 0) {
-	(void)write(fd, VICE_README, strlen(VICE_README));
+	len = strlen(VICE_README);
+	if (write(fd, VICE_README, len) != len)
+	    e = errno;
 	close(fd);
+	if (e)
+	    errno = e;
+
     }
     return (errno);
 }
@@ -669,26 +680,76 @@ SetOGM(FD_t fd, int parm, int tag)
 }
 
 static int
-CheckOGM(namei_t *name, FdHandle_t *fdP, int p1)
+SetWinOGM(FD_t fd, int p1, int p2)
 {
-    WIN32_FIND_DATA info;
-    HANDLE dirH;
+    BOOL code;
+    FILETIME ftime;
 
-    dirH =
-	FindFirstFileEx(name->n_path, FindExInfoStandard, &info,
-			FindExSearchNameMatch, NULL,
-			FIND_FIRST_EX_CASE_SENSITIVE);
+    ftime.dwHighDateTime = p1;
+    ftime.dwLowDateTime = p2;
 
-    if (!dirH)
-	return -1;          /* Can't get info, leave alone */
-
-    FindClose(dirH);
-
-    if (info.ftCreationTime.dwHighDateTime != (unsigned int)p1)
+    code = SetFileTime(fd, &ftime, NULL /*access*/, NULL /*write*/);
+    if (!code)
 	return -1;
+    return 0;
+}
+
+static int
+GetWinOGM(FD_t fd, int *p1, int *p2)
+{
+    BOOL code;
+    FILETIME ftime;
+
+    code = GetFileTime(fd, &ftime, NULL /*access*/, NULL /*write*/);
+    if (!code)
+	return -1;
+
+    *p1 = ftime.dwHighDateTime;
+    *p2 = ftime.dwLowDateTime;
 
     return 0;
 }
+
+static int
+CheckOGM(FdHandle_t *fdP, int p1)
+{
+    int ogm_p1, ogm_p2;
+
+    if (GetWinOGM(fdP->fd_fd, &ogm_p1, &ogm_p2)) {
+	return -1;
+    }
+
+    if (ogm_p1 != p1) {
+	return -1;
+    }
+
+    return 0;
+}
+
+static int
+FixSpecialOGM(FdHandle_t *fdP, int check)
+{
+    Inode ino = fdP->fd_ih->ih_ino;
+    VnodeId vno = NAMEI_VNODESPECIAL, ogm_vno;
+    int ogm_volid;
+
+    if (GetWinOGM(fdP->fd_fd, &ogm_volid, &ogm_vno)) {
+	return -1;
+    }
+
+    /* the only thing we can check is the vnode number; for the volid we have
+     * nothing else to compare against */
+    if (vno != ogm_vno) {
+	if (check) {
+	    return -1;
+	}
+	if (SetWinOGM(fdP->fd_fd, ogm_volid, vno)) {
+	    return -1;
+	}
+    }
+    return 0;
+}
+
 #else /* AFS_NT40_ENV */
 /* SetOGM - set owner group and mode bits from parm and tag */
 static int
@@ -723,21 +784,89 @@ GetOGMFromStat(struct afs_stat *status, int *parm, int *tag)
 }
 
 static int
-CheckOGM(namei_t *name, FdHandle_t *fdP, int p1)
+GetOGM(FdHandle_t *fdP, int *parm, int *tag)
 {
     struct afs_stat status;
-    int parm, tag;
     if (afs_fstat(fdP->fd_fd, &status) < 0)
 	return -1;
+    GetOGMFromStat(&status, parm, tag);
+    return 0;
+}
 
-    GetOGMFromStat(&status, &parm, &tag);
+static int
+CheckOGM(FdHandle_t *fdP, int p1)
+{
+    int parm, tag;
+
+    if (GetOGM(fdP, &parm, &tag) < 0)
+	return -1;
     if (parm != p1)
 	return -1;
 
     return 0;
 }
+
+static int
+FixSpecialOGM(FdHandle_t *fdP, int check)
+{
+    int inode_volid, ogm_volid;
+    int inode_type, ogm_type;
+    Inode ino = fdP->fd_ih->ih_ino;
+
+    inode_volid = ((ino >> NAMEI_UNIQSHIFT) & NAMEI_UNIQMASK);
+    inode_type = (int)((ino >> NAMEI_TAGSHIFT) & NAMEI_TAGMASK);
+
+    if (GetOGM(fdP, &ogm_volid, &ogm_type) < 0) {
+	Log("Error retrieving OGM info\n");
+	return -1;
+    }
+
+    if (inode_volid != ogm_volid || inode_type != ogm_type) {
+	Log("%sIncorrect OGM data (ino: vol %u type %d) (ogm: vol %u type %d)\n",
+	    check?"":"Fixing ", inode_volid, inode_type, ogm_volid, ogm_type);
+
+	if (check) {
+	    return -1;
+	}
+
+	if (SetOGM(fdP->fd_fd, inode_volid, inode_type) < 0) {
+	    Log("Error setting OGM data\n");
+	    return -1;
+	}
+    }
+    return 0;
+}
+
 #endif /* !AFS_NT40_ENV */
 
+/**
+ * Check/fix the OGM data for an inode
+ *
+ * @param[in] fdP   Open file handle for the inode to check
+ * @param[in] check 1 to just check the OGM data, and return an error if it
+ *                  is incorrect. 0 to fix the OGM data if it is incorrect.
+ *
+ * @pre fdP must be for a special inode
+ *
+ * @return status
+ *  @retval 0 success
+ *  @retval -1 error
+ */
+int
+namei_FixSpecialOGM(FdHandle_t *fdP, int check)
+{
+    int vnode;
+    Inode ino = fdP->fd_ih->ih_ino;
+
+    vnode = (int)(ino & NAMEI_VNODEMASK);
+    if (vnode != NAMEI_VNODESPECIAL) {
+	Log("FixSpecialOGM: non-special vnode %u\n", vnode);
+	return -1;
+    }
+
+    return FixSpecialOGM(fdP, check);
+}
+
 int big_vno = 0;		/* Just in case we ever do 64 bit vnodes. */
 
 /* Derive the name and create it O_EXCL. If that fails we have an error.
@@ -755,7 +884,7 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     FdHandle_t *fdP;
     FdHandle_t tfd;
     int type, tag;
-    FILETIME ftime;
+    int ogm_p1, ogm_p2;
     char *p;
     b32_string_t str1;
 
@@ -775,8 +904,8 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
 	 * p3 - type
 	 * p4 - parent volume id
 	 */
-        ftime.dwHighDateTime = p1;
-        ftime.dwLowDateTime = p2;
+        ogm_p1 = p1;
+        ogm_p2 = p2;
 	type = p3;
 	tmp.ih_vid = p4;	/* Use parent volume id, where this file will be. */
 	tmp.ih_ino = namei_MakeSpecIno(p1, p3);
@@ -797,8 +926,8 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
 
 	tmp.ih_vid = p1;
 	tmp.ih_ino = (Inode) p2;
-	ftime.dwHighDateTime = p3;
-        ftime.dwLowDateTime = p4;
+	ogm_p1 = p3;
+        ogm_p2 = p4;
     }
 
     namei_HandleToName(&name, &tmp);
@@ -806,11 +935,11 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     p++;
     for (tag = 0; tag < NAMEI_MAXVOLS; tag++) {
         *p = *int_to_base32(str1, tag);
-        fd = afs_open((char *)&name.n_path, O_CREAT | O_RDWR | O_TRUNC | O_EXCL, 0666);
+        fd = afs_open((char *)&name.n_path, O_CREAT | O_RDWR | O_EXCL, 0666);
         if (fd == INVALID_FD) {
             if (errno == ENOTDIR || errno == ENOENT) {
                 if (namei_CreateDataDirectories(&name, &created_dir) == 0)
-                    fd = afs_open((char *)&name.n_path, O_CREAT | O_RDWR | O_TRUNC | O_EXCL, 0666);
+                    fd = afs_open((char *)&name.n_path, O_CREAT | O_RDWR | O_EXCL, 0666);
             }
         }
 
@@ -827,7 +956,7 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     tmp.ih_ino |= (((Inode) tag) << NAMEI_TAGSHIFT);
 
     if (!code) {
-        if (!SetFileTime((HANDLE) fd, &ftime, NULL, NULL)) {
+        if (SetWinOGM(fd, ogm_p1, ogm_p2)) {
 	    errno = OS_ERROR(EBADF);
             code = -1;
         }
@@ -880,7 +1009,8 @@ bad:
 }
 #else /* !AFS_NT40_ENV */
 Inode
-namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4)
+icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4,
+        FD_t *afd, Inode *ainode)
 {
     namei_t name;
     int fd = INVALID_FD;
@@ -942,12 +1072,12 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     }
 
     namei_HandleToName(&name, &tmp);
-    fd = afs_open(name.n_path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR, 0);
+    fd = afs_open(name.n_path, O_CREAT | O_EXCL | O_RDWR, 0);
     if (fd < 0) {
 	if (errno == ENOTDIR || errno == ENOENT) {
 	    if (namei_CreateDataDirectories(&name, &created_dir) < 0)
 		goto bad;
-	    fd = afs_open(name.n_path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR,
+	    fd = afs_open(name.n_path, O_CREAT | O_EXCL | O_RDWR,
 			  0);
 	    if (fd < 0)
 		goto bad;
@@ -970,10 +1100,6 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     }
 
   bad:
-    if (fd >= 0)
-	close(fd);
-
-
     if (code || (fd < 0)) {
 	if (p2 != -1) {
 	    fdP = IH_OPEN(lh);
@@ -983,7 +1109,56 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
 	    }
 	}
     }
-    return (code || (fd < 0)) ? (Inode) - 1 : tmp.ih_ino;
+
+    *afd = fd;
+    *ainode = tmp.ih_ino;
+
+    return code;
+}
+
+Inode
+namei_icreate(IHandle_t * lh, char *part,
+              afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4)
+{
+    Inode ino = 0;
+    int fd = INVALID_FD;
+    int code;
+
+    code = icreate(lh, part, p1, p2, p3, p4, &fd, &ino);
+    if (fd >= 0) {
+	close(fd);
+    }
+    return (code || (fd < 0)) ? (Inode) - 1 : ino;
+}
+
+IHandle_t *
+namei_icreate_init(IHandle_t * lh, int dev, char *part,
+                   afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4)
+{
+    Inode ino = 0;
+    int fd = INVALID_FD;
+    int code;
+    IHandle_t *ihP;
+    FdHandle_t *fdP;
+
+    code = icreate(lh, part, p1, p2, p3, p4, &fd, &ino);
+    if (fd == INVALID_FD) {
+	return NULL;
+    }
+    if (code) {
+	close(fd);
+	return NULL;
+    }
+
+    IH_INIT(ihP, dev, p1, ino);
+    fdP = ih_attachfd(ihP, fd);
+    if (!fdP) {
+	close(fd);
+    } else {
+	FDH_CLOSE(fdP);
+    }
+
+    return ihP;
 }
 #endif
 
@@ -1028,7 +1203,7 @@ namei_dec(IHandle_t * ih, Inode ino, int p1)
 	    return -1;
 	}
 
-	if (CheckOGM(&name, fdP, p1) < 0) {
+	if (CheckOGM(fdP, p1) < 0) {
 	    FDH_REALLYCLOSE(fdP);
 	    IH_RELEASE(tmp);
 	    errno = OS_ERROR(EINVAL);
@@ -1063,6 +1238,11 @@ namei_dec(IHandle_t * ih, Inode ino, int p1)
 		/* Try to remove directory. If it fails, that's ok.
 		 * Salvage will clean up.
 		 */
+		char *slash = strrchr(name.n_path, OS_DIRSEPC);
+		if (slash) {
+		    /* avoid an rmdir() on the file we just unlinked */
+		    *slash = '\0';
+		}
 		(void)namei_RemoveDataDirectories(&name);
 	    }
 	}
@@ -1193,7 +1373,7 @@ namei_copy_on_write(IHandle_t *h)
 	if (!fdP)
 	    return EIO;
 	afs_snprintf(path, sizeof(path), "%s-tmp", name.n_path);
-	fd = afs_open(path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR, 0);
+	fd = afs_open(path, O_CREAT | O_EXCL | O_RDWR, 0);
 	if (fd < 0) {
 	    FDH_CLOSE(fdP);
 	    return EIO;
@@ -1422,6 +1602,9 @@ namei_GetLinkCount(FdHandle_t * h, Inode ino, int lockit, int fixup, int nowrite
     }
 
     rc = FDH_PREAD(h, (char*)&row, sizeof(row), offset);
+    if (rc == -1)
+	goto bad_getLinkByte;
+
     if ((rc == 0 || !((row >> index) & NAMEI_TAGMASK)) && fixup && nowrite)
         return 1;
     if (rc == 0 && fixup) {
@@ -1436,7 +1619,10 @@ namei_GetLinkCount(FdHandle_t * h, Inode ino, int lockit, int fixup, int nowrite
 	    NAMEI_GLC_UNLOCK;
 	    goto bad_getLinkByte;
 	}
-        FDH_TRUNC(h, offset+sizeof(row));
+        if (FDH_TRUNC(h, offset+sizeof(row))) {
+	    NAMEI_GLC_UNLOCK;
+	    goto bad_getLinkByte;
+	}
         row = 1 << index;
 	rc = FDH_PWRITE(h, (char *)&row, sizeof(row), offset);
 	NAMEI_GLC_UNLOCK;
@@ -1592,7 +1778,7 @@ namei_SetLinkCount(FdHandle_t * fdP, Inode ino, int count, int locked)
 
 /* ListViceInodes - write inode data to a results file. */
 static int DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
-		       unsigned int volid);
+		       IHandle_t *myIH);
 static int DecodeVolumeName(char *name, unsigned int *vid);
 static int namei_ListAFSSubDirs(IHandle_t * dirIH,
 				int (*write_fun) (FILE *,
@@ -1863,31 +2049,27 @@ _namei_examine_special(char * path1,
 {
     int ret = 0;
     struct ViceInodeInfo info;
-    afs_uint32 inode_vgid;
 
-    if (DecodeInode(path1, dname, &info, myIH->ih_vid) < 0) {
+    if (DecodeInode(path1, dname, &info, myIH) < 0) {
 	ret = 0;
 	goto error;
     }
 
-#ifdef AFS_NT40_ENV
-    inode_vgid = myIH->ih_vid;
-#else
-    inode_vgid = (info.inodeNumber >> NAMEI_UNIQSHIFT) & NAMEI_UNIQMASK;
-#endif
-
     if (info.u.param[2] != VI_LINKTABLE) {
 	info.linkCount = 1;
-    } else if ((info.u.param[0] != myIH->ih_vid) ||
-	       (inode_vgid != myIH->ih_vid)) {
+    } else if (info.u.param[0] != myIH->ih_vid) {
 	/* VGID encoded in linktable filename and/or OGM data isn't
 	 * consistent with VGID encoded in namei path */
 	Log("namei_ListAFSSubDirs: warning: inconsistent linktable "
 	    "filename \"%s" OS_DIRSEP "%s\"; salvager will delete it "
-	    "(dir_vgid=%u, inode_vgid=%u, ogm_vgid=%u)\n",
+	    "(dir_vgid=%u, inode_vgid=%u)\n",
 	    path1, dname, myIH->ih_vid,
-	    (unsigned int)inode_vgid,
 	    info.u.param[0]);
+	/* We need to set the linkCount to _something_, so linkCount
+	 * doesn't just contain stack garbage. Set it to 0, so in case
+	 * the salvager or whatever our caller is does try to process
+	 * this like a normal file, we won't try to INC or DEC it. */
+	info.linkCount = 0;
     } else {
 	char path2[512];
 	/* Open this handle */
@@ -1900,9 +2082,12 @@ _namei_examine_special(char * path1,
 
     if (!judgeFun ||
 	(*judgeFun) (&info, singleVolumeNumber, rock)) {
-	ret = 1;
-        if ((*writeFun) (fp, &info, path1, dname) < 0) {
+	ret = (*writeFun) (fp, &info, path1, dname);
+	if (ret < 0) {
+	    Log("_namei_examine_special: writeFun returned %d\n", ret);
 	    ret = -1;
+	} else {
+	    ret = 1;
 	}
     }
 
@@ -1956,7 +2141,7 @@ _namei_examine_reg(char * path3,
     int dirl; /* Windows-only (one level hash dir) */
 #endif
 
-    if (DecodeInode(path3, dname, &info, myIH->ih_vid) < 0) {
+    if (DecodeInode(path3, dname, &info, myIH) < 0) {
 	goto error;
     }
 
@@ -1982,9 +2167,12 @@ _namei_examine_reg(char * path3,
 
     if (!judgeFun ||
 	(*judgeFun) (&info, singleVolumeNumber, rock)) {
-	ret = 1;
-        if ((*writeFun) (fp, &info, path3, dname) < 0) {
+	ret = (*writeFun) (fp, &info, path3, dname);
+	if (ret < 0) {
+	    Log("_namei_examine_reg: writeFun returned %d\n", ret);
 	    ret = -1;
+	} else {
+	    ret = 1;
 	}
     }
 
@@ -2611,7 +2799,7 @@ DecodeVolumeName(char *name, unsigned int *vid)
 #ifdef AFS_NT40_ENV
 static int
 DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
-	    unsigned int volid)
+	    IHandle_t *myIH)
 {
     char fpath[512];
     int tag, vno;
@@ -2621,6 +2809,7 @@ DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
     char stmp[16];
     FdHandle_t linkHandle;
     char dirl;
+    VolumeId volid = myIH->ih_vid;
 
     afs_snprintf(fpath, sizeof(fpath), "%s" OS_DIRSEP "%s", dpath, name);
 
@@ -2687,12 +2876,16 @@ DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
 #else
 static int
 DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
-	    unsigned int volid)
+	    IHandle_t *myIH)
 {
     char fpath[512];
     struct afs_stat status;
+    struct afs_stat checkstatus;
     int parm, tag;
     lb64_string_t check;
+    VolumeId volid = myIH->ih_vid;
+    IHandle_t tmpih;
+    namei_t nameiname;
 
     afs_snprintf(fpath, sizeof(fpath), "%s" OS_DIRSEP "%s", dpath, name);
 
@@ -2707,14 +2900,47 @@ DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
     if (strcmp(name, check))
 	return -1;
 
-    GetOGMFromStat(&status, &parm, &tag);
+    /* Check if the _full_ path is correct, to ensure we can actually open this
+     * file later. Otherwise, the salvager can choke. */
+    memset(&tmpih, 0, sizeof(tmpih));
+    tmpih.ih_dev = myIH->ih_dev;
+    tmpih.ih_vid = myIH->ih_vid;
+    tmpih.ih_ino = info->inodeNumber;
+    namei_HandleToName(&nameiname, &tmpih);
+    if ((afs_stat(nameiname.n_path, &checkstatus) < 0) ||
+        checkstatus.st_ino != status.st_ino ||
+        checkstatus.st_size != status.st_size) {
+	static int logged;
+	/* log something for this case, since this means the filename looks
+	 * like a valid inode, but it's just in the wrong place. That's pretty
+	 * strange. */
+	if (!logged) {
+	    logged = 1;
+	    Log("Note:\n");
+	    Log("  Seemingly-misplaced files have been found, which I am\n");
+	    Log("  ignoring for now. If you cannot salvage the relevant volume,\n");
+	    Log("  you may try manually moving them to their correct location.\n");
+	    Log("  If the relevant volume seems fine, and these files do not\n");
+	    Log("  appear to contain important data, you can probably manually\n");
+	    Log("  delete them, or leave them alone. Contact your local OpenAFS\n");
+	    Log("  expert if you are unsure.\n");
+	}
+	Log("Ignoring misplaced file in volume group %u: %s (should be %s)\n",
+	    (unsigned)myIH->ih_vid, fpath, nameiname.n_path);
+	return -1;
+    }
+
     if ((info->inodeNumber & NAMEI_INODESPECIAL) == NAMEI_INODESPECIAL) {
+	parm = ((info->inodeNumber >> NAMEI_UNIQSHIFT) & NAMEI_UNIQMASK);
+	tag = (int)((info->inodeNumber >> NAMEI_TAGSHIFT) & NAMEI_TAGMASK);
+
 	/* p1 - vid, p2 - -1, p3 - type, p4 - rwvid */
 	info->u.param[0] = parm;
 	info->u.param[1] = -1;
 	info->u.param[2] = tag;
 	info->u.param[3] = volid;
     } else {
+	GetOGMFromStat(&status, &parm, &tag);
 	/* p1 - vid, p2 - vno, p3 - uniq, p4 - dv */
 	info->u.param[0] = volid;
 	info->u.param[1] = (int)(info->inodeNumber & NAMEI_VNODEMASK);
@@ -2753,6 +2979,20 @@ convertVolumeInfo(FD_t fdr, FD_t fdw, afs_uint32 vid)
     vd.inUse = 0;
     vd.uniquifier += 5000;	/* just in case there are still file copies from
 				 * the old RW volume around */
+
+    /* For ROs, the copyDate contains the time that the RO volume was actually
+     * created, and the creationDate just contains the last time the RO was
+     * copied from the RW data. So, make the new RW creationDate more accurate
+     * by setting it to copyDate, if copyDate is older. Since, we know the
+     * volume is at least as old as copyDate. */
+    if (vd.copyDate < vd.creationDate) {
+	vd.creationDate = vd.copyDate;
+    } else {
+	/* If copyDate is newer, just make copyDate and creationDate the same,
+	 * for consistency with other RWs */
+	vd.copyDate = vd.creationDate;
+    }
+
     p = strrchr(vd.name, '.');
     if (p && !strcmp(p, ".readonly")) {
 	memset(p, 0, 9);
@@ -2812,6 +3052,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     struct DiskPartition64 *partP;
     struct ViceInodeInfo info;
     struct VolumeDiskHeader h;
+    char *rwpart, *rwname;
+    Error ec;
 # ifdef AFS_DEMAND_ATTACH_FS
     int locktype = 0;
 # endif /* AFS_DEMAND_ATTACH_FS */
@@ -2841,6 +3083,19 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 	goto done;
     }
 
+    /* check for existing RW on any partition on this server; */
+    /* if found, return EXDEV - invalid cross-device link */
+    VOL_LOCK;
+    VGetVolumePath(&ec, h.parent, &rwpart, &rwname);
+    if (ec == 0) {
+	Log("1 namei_ConvertROtoRWvolume: RW volume %lu already exists on server partition %s.\n",
+	    afs_printable_uint32_lu(h.parent), rwpart);
+	code = EXDEV;
+	VOL_UNLOCK;
+	goto done;
+    }
+    VOL_UNLOCK;
+
     FSYNC_VolOp(volumeId, pname, FSYNC_VOL_BREAKCBKS, 0, NULL);
 
     ino = namei_MakeSpecIno(h.parent, VI_LINKTABLE);
@@ -2863,7 +3118,7 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 	if (*dp->d_name == '.')
 	    continue;
 #endif
-	if (DecodeInode(dir_name, dp->d_name, &info, ih->ih_vid) < 0) {
+	if (DecodeInode(dir_name, dp->d_name, &info, ih) < 0) {
 	    Log("1 namei_ConvertROtoRWvolume: DecodeInode failed for %s" OS_DIRSEP "%s\n",
 		dir_name, dp->d_name);
 	    closedir(dirp);
@@ -2934,7 +3189,7 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     }
     t_ih.ih_ino = namei_MakeSpecIno(ih->ih_vid, VI_VOLINFO);
     namei_HandleToName(&n, &t_ih);
-    fd2 = afs_open(n.n_path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR, 0);
+    fd2 = afs_open(n.n_path, O_CREAT | O_EXCL | O_RDWR, 0);
     if (fd2 == INVALID_FD) {
 	Log("1 namei_ConvertROtoRWvolume: could not create RW info file: %s\n", n.n_path);
 	OS_CLOSE(fd);
@@ -2967,7 +3222,11 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 #ifdef AFS_NT40_ENV
     MoveFileEx(n.n_path, newpath, MOVEFILE_WRITE_THROUGH);
 #else
-    link(newpath, n.n_path);
+    if (link(newpath, n.n_path)) {
+	Log("1 namei_ConvertROtoRWvolume: could not move SmallIndex file: %s\n", n.n_path);
+	code = -1;
+	goto done;
+    }
     OS_UNLINK(newpath);
 #endif
 
@@ -2986,7 +3245,11 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 #ifdef AFS_NT40_ENV
     MoveFileEx(n.n_path, newpath, MOVEFILE_WRITE_THROUGH);
 #else
-    link(newpath, n.n_path);
+    if (link(newpath, n.n_path)) {
+	Log("1 namei_ConvertROtoRWvolume: could not move LargeIndex file: %s\n", n.n_path);
+	code = -1;
+	goto done;
+    }
     OS_UNLINK(newpath);
 #endif
 
